from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _


@apphook_pool.register
class LeprikonApp(CMSApp):
    name = _('Leprikon')
    app_name = 'leprikon'

    def get_urls(self, page=None, language=None, **kwargs):
        return ['leprikon.urls']



# Generated by Django 1.9.13 on 2018-04-03 20:12

from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields
import leprikon.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0011_subjectregistration_payment_requested'),
    ]

    operations = [
        migrations.CreateModel(
            name='SubjectVariant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='variant name')),
                ('description', djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='variant description')),
                ('price', leprikon.models.fields.PriceField(blank=True, decimal_places=0, max_digits=10, null=True, verbose_name='price')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('subject', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='variants', to='leprikon.Subject', verbose_name='subject')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'variant',
                'verbose_name_plural': 'variants',
            },
        ),
        migrations.AddField(
            model_name='subjectregistration',
            name='subject_variant',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='registrations', to='leprikon.SubjectVariant', verbose_name='variant'),
        ),
    ]



from django import forms
from django.core.exceptions import ValidationError
from django.utils import formats
from django.utils.translation import ugettext_lazy as _

from ..models.timesheets import Timesheet, TimesheetEntry
from .fields import ReadonlyField
from .form import FormMixin


class TimesheetEntryAdminForm(forms.ModelForm):
    """
    Validation of ManyToManyField must be performed in form.clean()
    Always use TimesheetEntryAdminForm to change TimesheetEntry
    """

    class Meta:
        model   = TimesheetEntry
        fields  = ('date', 'start', 'end', 'entry_type', 'description')

    def __init__(self, **kwargs):
        self.leader = kwargs.pop('leader', None) or kwargs['instance'].timesheet.leader
        super(TimesheetEntryAdminForm, self).__init__(**kwargs)

        # make date, start and end read only, if timesheet is already submitted
        if self.instance.id and self.instance.timesheet.submitted:
            self.readonly = True
            self.readonly_fields = [
                ReadonlyField(label=_('Date'),          value=self.instance.date),
                ReadonlyField(label=_('Start'),         value=self.instance.start),
                ReadonlyField(label=_('End'),           value=self.instance.end),
                ReadonlyField(label=_('Entry type'),    value=self.instance.entry_type),
            ]
            self.fields = {'description': self.fields['description']}
        else:
            self.readonly = False

    def clean_date(self):
        if self.instance.id:
            if self.cleaned_data['date'] < self.instance.timesheet.period.start:
                raise ValidationError(_('The timesheet period {period} starts {start}').format(
                    period  = self.instance.timesheet.period.name,
                    start   = formats.date_format(self.instance.timesheet.period.start, "SHORT_DATE_FORMAT"),
                ))
            if self.cleaned_data['date'] > self.instance.timesheet.period.end:
                raise ValidationError(_('The timesheet period {period} ends {end}').format(
                    period  = self.instance.timesheet.period.name,
                    end   = formats.date_format(self.instance.timesheet.period.end, "SHORT_DATE_FORMAT"),
                ))
        else:
            self.instance.timesheet = Timesheet.objects.for_leader_and_date(leader=self.leader,
                                                                            date=self.cleaned_data['date'])
            if self.instance.timesheet.submitted:
                raise ValidationError(_('Your timesheet for period {period} has already been submitted.').format(
                    period  = self.instance.timesheet.period.name,
                ))
        return self.cleaned_data['date']

    def clean(self):
        # check overlaping entries
        if 'date' in self.cleaned_data and 'start' in self.cleaned_data and 'end' in self.cleaned_data:
            qs = self.instance.timesheet.timesheet_entries.filter(
                date        = self.cleaned_data['date'],
                start__lt   = self.cleaned_data['end'],
                end__gt     = self.cleaned_data['start'],
            )
            if self.instance.id:
                qs = qs.exclude(id=self.instance.id)
            if qs.exists():
                raise ValidationError(_('An overlaping entry has already been added in the timesheet.'))
        return self.cleaned_data



class TimesheetEntryForm(FormMixin, TimesheetEntryAdminForm):
    pass



from django import forms
from django.contrib.auth import get_user_model, password_validation
from django.contrib.auth.forms import (
    AuthenticationForm as _AuthenticationForm,
    PasswordChangeForm as _PasswordChangeForm,
    PasswordResetForm as _PasswordResetForm,
    SetPasswordForm as _SetPasswordForm, UserCreationForm as _UserCreationForm,
)
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from verified_email_field.forms import VerifiedEmailField

from .form import FormMixin

User = get_user_model()


class UserFormMixin(FormMixin):

    def clean_email(self):
        user_qs = User.objects.filter(email=self.cleaned_data['email'])
        if self.instance.pk:
            user_qs = user_qs.exclude(pk=self.instance.pk)
        if self.cleaned_data['email'] and user_qs.exists():
            raise ValidationError(
                _('User with this email already exists.'),
                code='exists',
                params={'email': self.cleaned_data['email']},
            )
        else:
            return self.cleaned_data['email']


class UserAdminCreateForm(UserFormMixin, forms.ModelForm):

    # create user with random password
    # users without password may not reset the password themselves
    def save(self, commit=True):
        user = super(UserAdminCreateForm, self).save(commit)
        user.set_password(User.objects.make_random_password())
        user.save()
        return user

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', 'email']


class UserCreateForm(UserFormMixin, _UserCreationForm):
    email = VerifiedEmailField(label=_('E-mail'), fieldsetup_id='UserCreateForm', required=True)
    agreement = forms.BooleanField(label=_('I agree with the terms.'), required=True)

    def __init__(self, **kwargs):
        super(UserCreateForm, self).__init__(**kwargs)
        self.fields['username'].help_text = None
        self.fields['password1'].help_text = password_validation.password_validators_help_text_html()

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', 'email']


class UserAgreementForm(FormMixin, forms.Form):
    agreement = forms.BooleanField(label=_('I agree with the terms.'), required=True)


class UserEmailForm(UserFormMixin, forms.ModelForm):
    email = VerifiedEmailField(label='email', fieldsetup_id='UserEmailForm', required=True)

    class Meta:
        model = User
        fields = ['email']


class UserUpdateForm(UserFormMixin, forms.ModelForm):

    def __init__(self, **kwargs):
        super(UserUpdateForm, self).__init__(**kwargs)
        self.fields['username'].help_text = None

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name']


class UserLoginForm(FormMixin, _AuthenticationForm):
    pass


class UserPasswordForm(FormMixin, _PasswordChangeForm):
    pass


class PasswordResetForm(FormMixin, _PasswordResetForm):
    pass


class SetPasswordForm(FormMixin, _SetPasswordForm):
    pass



# Generated by Django 1.11.12 on 2018-08-29 09:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0015_school_year_divisions'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='course',
            name='periods',
        ),
        migrations.RemoveField(
            model_name='course',
            name='unit',
        ),
        migrations.RemoveField(
            model_name='schoolyearperiod',
            name='school_year',
        ),
        migrations.AlterField(
            model_name='course',
            name='school_year_division',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='courses', to='leprikon.SchoolYearDivision', verbose_name='school year division'),
        ),
        migrations.AlterField(
            model_name='schoolyearperiod',
            name='school_year_division',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='periods', to='leprikon.SchoolYearDivision', verbose_name='school year division'),
        ),
    ]



from haystack import indexes


class BaseIndex(indexes.SearchIndex, indexes.Indexable):
    text        = indexes.CharField(document=True, use_template=True, template_name='leprikon/search.txt')
    title       = indexes.CharField(stored=True, indexed=False, model_attr='name')
    url         = indexes.CharField(stored=True, indexed=False)
    school_year = indexes.IntegerField(stored=True, indexed=True, model_attr='school_year_id')

    def prepare_url(self, obj):
        return obj.get_absolute_url()



from . import (  # noqa
    agegroup, citizenship, courses, department, events, fields, leprikonsite,
    messages, place, printsetup, question, roles, school, schoolyear, startend,
    subjects, timesheets, useragreement, utils,
)



from collections import namedtuple

from django.core.urlresolvers import reverse_lazy as reverse
from django.template.response import TemplateResponse
from django.utils.translation import ugettext_lazy as _

from . import ReportBaseView
from ...forms.reports.debtors import DebtorsForm
from ...models.courses import CourseRegistration
from ...models.events import EventRegistration


class ReportDebtorsView(ReportBaseView):
    form_class      = DebtorsForm
    template_name   = 'leprikon/reports/debtors.html'
    title           = _('Debtors list')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    ReportItem  = namedtuple('ReportItem', ('registration', 'balance'))

    class Report(list):
        def append(self, item):
            super(ReportDebtorsView.Report, self).append(item)
            self.balance = getattr(self, 'balance', 0) + item.balance

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['reports'] = {}
        context['sum'] = 0

        for reg in CourseRegistration.objects.filter(subject__school_year = self.request.school_year,
                                                     created__date__lte = context['date']):
            balance = reg.get_payment_statuses(context['date']).partial.balance
            if balance < 0:
                report = context['reports'].setdefault(reg.user, self.Report())
                report.append(self.ReportItem(registration=reg, balance=balance))
                context['sum'] += balance

        for reg in EventRegistration.objects.filter(subject__school_year = self.request.school_year,
                                                    created__date__lte = context['date']):
            balance = reg.get_payment_status(context['date']).balance
            if balance < 0:
                report = context['reports'].setdefault(reg.user, self.Report())
                report.append(self.ReportItem(registration=reg, balance=balance))
                context['sum'] += balance

        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



from django.contrib.sites.models import Site, clear_site_cache
from django.db import models
from django.db.models.signals import pre_delete, pre_save
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField
from localflavor.generic.models import BICField, IBANField

from .fields import PostalCodeField
from .printsetup import PrintSetup
from .utils import BankAccount


class LeprikonSiteManager(models.Manager):

    def get_current(self, request=None):
        current_site = Site.objects.get_current(request)
        return self.get_or_create(site_ptr=current_site)[0]


class LeprikonSite(Site):
    company_name    = models.CharField(_('company name'), max_length=150, blank=True, null=True)
    street          = models.CharField(_('street'), max_length=150, blank=True, null=True)
    city            = models.CharField(_('city'), max_length=150, blank=True, null=True)
    postal_code     = PostalCodeField(_('postal code'), blank=True, null=True)
    email           = models.EmailField(_('email address'), blank=True, null=True)
    phone           = models.CharField(_('phone'), max_length=30, blank=True, null=True)
    company_num     = models.CharField(_('company number'), max_length=8, blank=True, null=True)
    vat_number      = models.CharField(_('VAT number'), max_length=10, blank=True, null=True)
    iban            = IBANField(_('IBAN'), blank=True, null=True)
    bic             = BICField(_('BIC (SWIFT)'), blank=True, null=True)
    bill_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                         verbose_name=_('bill print setup'), blank=True, null=True)
    registration_agreement  = HTMLField(_('registration agreement'), blank=True, default='')
    reg_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                        verbose_name=_('registration print setup'), blank=True, null=True)
    user_agreement  = HTMLField(_('user agreement'), blank=True, default='')
    user_agreement_changed = models.DateTimeField(_('last time user agreement changed'), blank=True, null=True)

    objects = LeprikonSiteManager()

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('leprikon site')
        verbose_name_plural = _('leprikon sites')

    def get_company_name(self):
        return self.company_name or self.name

    @cached_property
    def bank_account(self):
        return self.iban and BankAccount(self.iban)


pre_save.connect(clear_site_cache, sender=LeprikonSite)
pre_delete.connect(clear_site_cache, sender=LeprikonSite)
